UNIT VGA;
{{$r-,q-,i-,x+,opt+,d-,l-}
{
This is the VGA part of the graphics subsystem.
------------------------------------------------
Current Version:
         0.02 - 18-02-99 by JacmeT (jacmet@iname.com)

Purpose: 
         To handle the communication between
         the program and the graphics card
         through the standard VGA interface (mode 13h).

Functionality:
         + mode 13h support
         + Scaling of virtual screen to 320x200

         (minus indicates not done yet, 
         plus indicates it is implemented and tested
         and question mark indicates implemented
         (perhaps partly) but not tested yet)
           
Comments:
        The use of defines are heavilly approved.
        As of this moment the following defines are
        defined:
        ALL           : All code is defined (Doesn't affect verbose level)
        VERBOSE_LEVEL0: Verbose level 0 - no messages are included
        VERBOSE_LEVEL1: Verbose level 1 - only the most important messages are included
        VERBOSE_LEVEL2: Verbose level 2 - most messages are included
        VERBOSE_LEVEL3: Verbose level 3 - all messages are included
        DEBUG         : Debugging code included
  
History:
         15-02-99: First revision (0.01)
         --------------------------------
         This is the very first version.
         At this point the Initialize
         function always returns true.
         In the future it will check if
         there's a VGA card installed.

         18-02-99: Second revision (0.02)
         --------------------------------
         Setmode procedure changed because
         of new scaling version of VGA mode
         flip procedure. Now the Setmode
         procedure calculates DeltaX and
         DeltaY (the amount of pixels to
         move in the virtual screen to get
         to the next pixel in 320x200) in
         24.8 bits fixed point.
         DeltaX and DeltaY variables added
         for Setmode procedure.
}

{$I VGA.INC}
{$I ..\GENERAL.INC}

INTERFACE

Function Init: Boolean;
{ The Initialization function.}
Procedure Setmode(X,Y: dword);
{ The Setmode function. }
Procedure DeInit;
{ The DeInitialization procedure.}

VAR DeltaX,DeltaY: dword;
IMPLEMENTATION


Function Init: Boolean;
{ The initialization function. 
  -----------------------------
Purpose:
         To detect if VGA mode is
         available.
Comments:
         This function must be called
         before any of the others -
         See also the DeInit procedure
         The function returns a boolean
         expression telling if the 
         function call was successful
         or not.
}         
 begin
  {$IFDEF VERBOSE_LEVEL2}
   Writeln('VGA: Initialization successful');
  {$ENDIF}  
  Init:=True;
 end; {Init}

Procedure Setmode(X,Y: dword);
{ The SetMode function. 
  -----------------------------
Purpose:
         This procedure sets mode 13h
         and changes the pallette to
         BBGGGRRR format (3bit Red,
         3bit Green and 2bit Blue)

Comments:
         The requested resolution (X,Y)
         isn't used to set a certain 
         screenmode, since the only
         256 color mode available in 
         a standard VGA bios is 
         320x200 (13h) - instead it is
         used for the scaling (see also
         FLIP.PAS).
}         
 var color: byte;
 begin
  {calculate DeltaX (with 24.8 bit fixed point)}
  DeltaX:=(X shl 8) div 320;
  {calculate DeltaY (with 24.8 bit fixed point)}
  DeltaY:=(Y shl 8) div 200;
  {set mode 13h (320x200x256)}
  asm
   mov ax,13h
   int 10h
  end;
  {change pallette to RRGGGBBB}
  for color:=0 to 255 do
   begin
    port[$3c8]:=color;
    port[$3c9]:=((color and 192) shr 2)+8;
    port[$3c9]:=(color and 56);
    port[$3c9]:=(color and 7) shl 3;
   {the extra 8 is to make the intensity of
    the blue part equal to the red and green}
   end;
 end; {SetMode}

Procedure DeInit;
{ The deinitialization function. 
  ------------------------------
Purpose:
         This procedure deinitializes
         and returns to standard mode
         3 textmode.
Comments:
        None.
}         
 begin
  asm
   mov ax,3
   int 10h
  end;
  {$IFDEF VERBOSE_LEVEL2}
   Writeln('VGA: DeInitialization successful');
  {$ENDIF}
 end; {DeInit}

BEGIN
END.